#ifndef romcalls_h
#define romcalls_h

#define NULL	0

typedef unsigned long size_t;
typedef unsigned char  BYTE;
typedef unsigned char  UCHAR;
typedef unsigned short BOOL;
#define TRUE 1
#define FALSE 0
typedef unsigned short WORD;
typedef unsigned short USHORT;
typedef unsigned int   UINT;
typedef unsigned long  DWORD;
typedef unsigned long  ULONG;
typedef signed char    SBYTE;
typedef signed char    SCHAR;
typedef signed short   SWORD;
typedef signed short   SSHORT;
typedef signed int     SINT;
typedef signed long    SDWORD;
typedef signed long    SLONG;

typedef unsigned short HANDLE;
typedef char* ESI;

#define ValidateRect(r) assert(r.xy.x0 <= MAX_X && r.xy.x1 <= MAX_X &&  \
r.xy.y0 <= MAX_Y && r.xy.y1 <= MAX_Y && r.xy.x0 <= r.xy.x1 && r.xy.y0 <= r.xy.y1 )
#define ValidateRectP(r) assert(r->xy.x0 <= MAX_X && r->xy.x1 <= MAX_X &&  \
r->xy.y0 <= MAX_Y && r->xy.y1 <= MAX_Y && r->xy.x0 <= r->xy.x1 && r->xy.y0 <= r->xy.y1 )
#define ValidateX(x) assert(x >= 0 && x <= MAX_X)
#define ValidateY(y) assert(y >= 0 && y <= MAX_Y)
#define ValidateFont(f) assert(f <= F_8x10)
typedef unsigned char SCR_COORDS;
#define BITMAP_HDR_SIZE   4

typedef union {
struct {
  SCR_COORDS x0, y0;
  SCR_COORDS x1, y1;
} xy;
unsigned long l;
} SCR_RECT;

typedef signed short WIN_COORDS;

typedef struct {unsigned short i[16];} ICON;
typedef unsigned short *pICON;


typedef struct {
  WORD NumRows;
  WORD NumCols;
  BYTE Data[0]; 	/* should be [] */
} BITMAP;

typedef struct {
  HANDLE folder;
  WORD offset;
} HSym;

typedef struct {
  char name[8];
  WORD padding;
  union {
    WORD flags_n;
    struct {
      unsigned int flag1_7 : 1;
      unsigned int local : 1;
      unsigned int flag1_5 : 1;
      unsigned int flag1_4 : 1;
      unsigned int flag1_3 : 1;
      unsigned int flag1_2 : 1;
      unsigned int archived : 1;
      unsigned int flag1_0 : 1;

      unsigned int folder : 1;
      unsigned int flag0_6 : 1;
      unsigned int flag0_5 : 1;
      unsigned int hidden : 1;
      unsigned int locked : 1;
      unsigned int flag0_2 : 1;
      unsigned int flag0_1 : 1;
      unsigned int flag0_0 : 1;
    } bits;
  } flags;
  HANDLE handle;
} SYM_ENTRY;

typedef struct {
  WIN_COORDS x0, y0;
  WIN_COORDS x1, y1;
} WIN_RECT;

typedef struct {
  WIN_COORDS x0, y0;
} WIN_POINT;

enum Attrs { A_REVERSE=0, A_NORMAL=1, A_XOR, A_SHADED, A_REPLACE, A_OR,
	     A_AND, A_THICK1, A_SHADE_V, A_SHADE_H, A_SHADE_NS, A_SHADE_PS };
enum BoxAddrs { B_NORMAL=0x10, B_ROUNDED=0x20, B_DOUBLE=0x40, B_CUT=0x80 };
enum Fonts { F_4x6, F_6x8, F_8x10 };

/*  Defines / Structures  */

#define WinEnd(w)
#define WinSetCursor(w,x,y) (w)->CursorX=x; (w)->CursorY=y
#define WinShow(w) (w)->Flags |= WF_VISIBLE

enum WinFlags { WF_SYS_ALLOC=0x0001, WF_STEAL_MEM=0x0002,
		WF_DONT_REALLOC=0x0004, WF_ROUNDEDBORDER=0x0008,
		WF_SAVE_SCR=0x0010, WF_DUP_SCR=0x0020, WF_TTY=0x0040,
		WF_ACTIVE=0x0080, WF_NOBORDER=0x0100, WF_NOBOLD=0x0200,
		WF_DUP_ON=0x0400, WF_VIRTUAL=0x0800, WF_TITLE=0x1000,
		WF_DIRTY=0x2000, WF_TRY_SAVE_SCR=0x4010, WF_VISIBLE=0x8000 };

typedef struct WindowStruct {
  union {
    WORD all;
    struct {
      unsigned int visible : 1;
      unsigned int try_save_scr : 1;
      unsigned int dirty : 1;
      unsigned int title : 1;
      unsigned int virtual : 1;
      unsigned int dup_on : 1;
      unsigned int nobold : 1;
      unsigned int noborder : 1;
      unsigned int active : 1;
      unsigned int tty : 1;
      unsigned int dup_scr : 1;
      unsigned int save_scr : 1;
      unsigned int roundedborder : 1;
      unsigned int dont_realloc : 1;
      unsigned int steal_mem : 1;
      unsigned int sys_alloc : 1;
    } bits;
  } Flags;
  BYTE CurFont; 	       /* Current font */
  BYTE CurAttr; 	       /* Current text/line attribute */
  BYTE Background;	       /* Background attribute */
  SINT TaskId;		       /* Task ID of owner */
  WIN_COORDS CurX, CurY;       /* Current X,Y position (window coordinates) */
  WIN_COORDS CursorX, CursorY; /* Cursor X,Y position */
  SCR_RECT Client;	       /* Client region of window (excludes border) */
  SCR_RECT Window;	       /* Window region (entire window inc border) */
  SCR_RECT Clip;	       /* Current clipping region */
  SCR_RECT Port;	       /* Port region for DUP_SCR */
  HANDLE DupScr;	       /* Duplicated or saved screen area */
  struct WindowStruct *Next;   /* Windows kept in linked list */
  char *Title;		       /* Optional title */
} WINDOW;


#define malloc HeapAllocPtr
#define free HeapFreePtr

#define FirstWindow 	_ROM_CALL_0
#define WinActivate 	_ROM_CALL_1
#define WinAttr 	_ROM_CALL_2
#define WinBackupToScr 	_ROM_CALL_3
#define WinBackground 	_ROM_CALL_4
#define WinBegin 	_ROM_CALL_5
#define WinBitmapGet 	_ROM_CALL_6
#define WinBitmapPut 	_ROM_CALL_7
#define WinBitmapSize 	_ROM_CALL_8
#define WinCharXY 	_ROM_CALL_9
#define WinChar 	_ROM_CALL_A
#define WinClose 	_ROM_CALL_B
#define WinClr 	_ROM_CALL_C
#define WinDeactivate 	_ROM_CALL_D
#define WinDupStat 	_ROM_CALL_E
#define WinEllipse 	_ROM_CALL_F
#define WinFill 	_ROM_CALL_10
#define WinFillLines2 	_ROM_CALL_11
#define WinFillTriangle 	_ROM_CALL_12
#define WinFont 	_ROM_CALL_13
#define WinGetCursor 	_ROM_CALL_14
#define WinHide 	_ROM_CALL_15
#define WinHome 	_ROM_CALL_16
#define WinLine 	_ROM_CALL_17
#define WinLineNC 	_ROM_CALL_18
#define WinLineTo 	_ROM_CALL_19
#define WinLineRel 	_ROM_CALL_1A
#define WinMoveCursor 	_ROM_CALL_1B
#define WinMoveTo 	_ROM_CALL_1C
#define WinMoveRel 	_ROM_CALL_1D
#define WinOpen 	_ROM_CALL_1E
#define WinPixGet 	_ROM_CALL_1F
#define WinPixSet 	_ROM_CALL_20
#define WinRect 	_ROM_CALL_21
#define WinReOpen 	_ROM_CALL_22
#define WinScrollH 	_ROM_CALL_23
#define WinScrollV 	_ROM_CALL_24
#define WinStr 	_ROM_CALL_25
#define WinStrXY 	_ROM_CALL_26
#define DrawWinBorder 	_ROM_CALL_27
#define ScrRectDivide 	_ROM_CALL_28
#define RectWinToWin 	_ROM_CALL_29
#define RectWinToScr 	_ROM_CALL_2A
#define UpdateWindows 	_ROM_CALL_2B
#define MakeWinRect 	_ROM_CALL_2C
#define ScrToWin 	_ROM_CALL_2D
#define ScrToHome 	_ROM_CALL_2E
#define ScrRect 	_ROM_CALL_2F
#define Dialog 	_ROM_CALL_30
#define NoCallBack 	_ROM_CALL_31
#define DialogDo 	_ROM_CALL_32
#define DialogAdd 	_ROM_CALL_33
#define DialogNew 	_ROM_CALL_34
#define DrawStaticButton 	_ROM_CALL_35
#define MenuBegin 	_ROM_CALL_36
#define MenuCheck 	_ROM_CALL_37
#define MenuEnd 	_ROM_CALL_38
#define MenuKey 	_ROM_CALL_39
#define MenuOn 	_ROM_CALL_3A
#define MenuPopup 	_ROM_CALL_3B
#define MenuSubStat 	_ROM_CALL_3C
#define MenuTopStat 	_ROM_CALL_3D
#define MenuTopSelect 	_ROM_CALL_3E
#define MenuTopRedef 	_ROM_CALL_3F
#define MenuGetTopRedef 	_ROM_CALL_40
#define MenuAddText 	_ROM_CALL_41
#define MenuAddIcon 	_ROM_CALL_42
#define MenuNew 	_ROM_CALL_43
#define PopupAddText 	_ROM_CALL_44
#define PopupNew 	_ROM_CALL_45
#define PopupClear 	_ROM_CALL_46
#define PopupDo 	_ROM_CALL_47
#define PopupText 	_ROM_CALL_48
#define MenuUpdate 	_ROM_CALL_49
#define Parse2DExpr 	_ROM_CALL_4A
#define Parse2DMultiExpr 	_ROM_CALL_4B
#define Print2DExpr 	_ROM_CALL_4C
#define Parms2D 	_ROM_CALL_4D
#define display_statements 	_ROM_CALL_4E
#define Parse1DExpr 	_ROM_CALL_4F
#define pushkey 	_ROM_CALL_50
#define ngetchx 	_ROM_CALL_51
#define kbhit 	_ROM_CALL_52
#define sprintf 	_ROM_CALL_53
#define getcalc 	_ROM_CALL_54
#define sendcalc 	_ROM_CALL_55
#define LIO_Send 	_ROM_CALL_56
#define LIO_Get 	_ROM_CALL_57
#define LIO_Receive 	_ROM_CALL_58
#define LIO_GetMultiple 	_ROM_CALL_59
#define LIO_SendData 	_ROM_CALL_5A
#define LIO_RecvData 	_ROM_CALL_5B
#define SymAdd 	_ROM_CALL_5C
#define SymAddMain 	_ROM_CALL_5D
#define SymDel 	_ROM_CALL_5E
#define HSymDel 	_ROM_CALL_5F
#define SymFind 	_ROM_CALL_60
#define SymFindMain 	_ROM_CALL_61
#define SymFindHome 	_ROM_CALL_62
#define SymMove 	_ROM_CALL_63
#define FolderAdd 	_ROM_CALL_64
#define FolderCur 	_ROM_CALL_65
#define FolderDel 	_ROM_CALL_66
#define FolderFind 	_ROM_CALL_67
#define FolderGetCur 	_ROM_CALL_68
#define FolderOp 	_ROM_CALL_69
#define FolderRename 	_ROM_CALL_6A
#define FolderCount 	_ROM_CALL_6B
#define SymFindFirst 	_ROM_CALL_6C
#define SymFindNext 	_ROM_CALL_6D
#define SymFindPrev 	_ROM_CALL_6E
#define SymFindFoldername 	_ROM_CALL_6F
#define AddSymToFolder 	_ROM_CALL_70
#define FindSymInFolder 	_ROM_CALL_71
#define FolderCurTemp 	_ROM_CALL_72
#define FolderAddTemp 	_ROM_CALL_73
#define FolderDelTemp 	_ROM_CALL_74
#define FolderDelAllTemp 	_ROM_CALL_75
#define TempFolderName 	_ROM_CALL_76
#define IsMainFolderStr 	_ROM_CALL_77
#define ParseSymName 	_ROM_CALL_78
#define DerefSym 	_ROM_CALL_79
#define HSYMtoName 	_ROM_CALL_7A
#define StrToTokN 	_ROM_CALL_7B
#define TokToStrN 	_ROM_CALL_7C
#define CheckGraphRef 	_ROM_CALL_7D
#define ClearUserDef 	_ROM_CALL_7E
#define CheckLinkLockFlag 	_ROM_CALL_7F
#define TokenizeSymName 	_ROM_CALL_80
#define SymCmp 	_ROM_CALL_81
#define SymCpy 	_ROM_CALL_82
#define SymCpy0 	_ROM_CALL_83
#define ValidateSymName 	_ROM_CALL_84
#define VarRecall 	_ROM_CALL_85
#define VarStore 	_ROM_CALL_86
#define VarStoreLink 	_ROM_CALL_87
#define QSysProtected 	_ROM_CALL_88
#define CheckSysFunc 	_ROM_CALL_89
#define GetSysGraphRef 	_ROM_CALL_8A
#define CheckReservedName 	_ROM_CALL_8B
#define SymSysVar 	_ROM_CALL_8C
#define ValidateStore 	_ROM_CALL_8D
#define ResetSymFlags 	_ROM_CALL_8E
#define HeapAvail 	_ROM_CALL_8F
#define HeapAlloc 	_ROM_CALL_90
#define HeapAllocESTACK 	_ROM_CALL_91
#define HeapAllocHigh 	_ROM_CALL_92
#define HeapAllocThrow 	_ROM_CALL_93
#define HeapAllocHighThrow 	_ROM_CALL_94
#define HeapCompress 	_ROM_CALL_95
#define HeapDeref 	_ROM_CALL_96
#define HeapFree 	_ROM_CALL_97
#define HeapFreeIndir 	_ROM_CALL_98
#define HLock 	_ROM_CALL_99
#define HeapLock 	_ROM_CALL_9A
#define HeapGetLock 	_ROM_CALL_9B
#define HeapMax 	_ROM_CALL_9C
#define HeapRealloc 	_ROM_CALL_9D
#define HeapSize 	_ROM_CALL_9E
#define HeapUnlock 	_ROM_CALL_9F
#define HeapMoveHigh 	_ROM_CALL_A0
#define HeapEnd 	_ROM_CALL_A1
#define HeapAllocPtr 	_ROM_CALL_A2
#define HeapFreePtr 	_ROM_CALL_A3
#define NeedStack 	_ROM_CALL_A4
#define TE_close 	_ROM_CALL_A5

#define TE_checkSlack 	_ROM_CALL_A6
#define TE_empty 	_ROM_CALL_A7
#define TE_focus 	_ROM_CALL_A8
#define TE_handleEvent 	_ROM_CALL_A9
#define TE_indicateReadOnly 	_ROM_CALL_AA
#define TE_isBlank 	_ROM_CALL_AB
#define TE_open 	_ROM_CALL_AC
#define TE_openFixed 	_ROM_CALL_AD
#define TE_pasteText 	_ROM_CALL_AE
#define TE_reopen 	_ROM_CALL_AF
#define TE_reopenPlain 	_ROM_CALL_B0
#define TE_select 	_ROM_CALL_B1
#define TE_shrinkWrap 	_ROM_CALL_B2
#define TE_unfocus 	_ROM_CALL_B3
#define TE_updateCommand 	_ROM_CALL_B4
#define _bcd_math 	_ROM_CALL_B5
#define bcdadd 	_ROM_CALL_B6
#define bcdsub 	_ROM_CALL_B7
#define bcdmul 	_ROM_CALL_B8
#define bcddiv 	_ROM_CALL_B9
#define bcdneg 	_ROM_CALL_BA
#define bcdcmp 	_ROM_CALL_BB
#define bcdlong 	_ROM_CALL_BC
#define bcdbcd 	_ROM_CALL_BD
#define EX_getArg 	_ROM_CALL_BE
#define EX_getBCD 	_ROM_CALL_BF
#define EX_stoBCD 	_ROM_CALL_C0
#define CB_replaceTEXT 	_ROM_CALL_C1
#define CB_fetchTEXT 	_ROM_CALL_C2
#define CU_restore 	_ROM_CALL_C3
#define CU_start 	_ROM_CALL_C4
#define CU_stop 	_ROM_CALL_C5
#define EV_captureEvents 	_ROM_CALL_C6
#define EV_clearPasteString 	_ROM_CALL_C7
#define EV_getc 	_ROM_CALL_C8
#define EV_getSplitRect 	_ROM_CALL_C9
#define EV_notifySwitchGraph 	_ROM_CALL_CA
#define EV_paintOneWindow 	_ROM_CALL_CB
#define EV_paintWindows 	_ROM_CALL_CC
#define EV_restorePainting 	_ROM_CALL_CD
#define EV_sendEvent 	_ROM_CALL_CE
#define EV_sendEventSide 	_ROM_CALL_CF
#define EV_sendString 	_ROM_CALL_D0
#define EV_setCmdCheck 	_ROM_CALL_D1
#define EV_setCmdState 	_ROM_CALL_D2
#define EV_setFKeyState 	_ROM_CALL_D3
#define EV_startApp 	_ROM_CALL_D4
#define EV_startSide 	_ROM_CALL_D5
#define EV_startTask 	_ROM_CALL_D6
#define EV_suspendPainting 	_ROM_CALL_D7
#define EV_switch 	_ROM_CALL_D8
#define MO_currentOptions 	_ROM_CALL_D9
#define MO_defaults 	_ROM_CALL_DA
#define MO_digestOptions 	_ROM_CALL_DB
#define MO_isMultigraphTask 	_ROM_CALL_DC
#define MO_modeDialog 	_ROM_CALL_DD
#define MO_notifyModeChange 	_ROM_CALL_DE
#define MO_sendQuit 	_ROM_CALL_DF
#define ST_angle 	_ROM_CALL_E0
#define ST_batt 	_ROM_CALL_E1
#define ST_busy 	_ROM_CALL_E2
#define ST_eraseHelp 	_ROM_CALL_E3
#define ST_folder 	_ROM_CALL_E4
#define ST_graph 	_ROM_CALL_E5
#define ST_helpMsg 	_ROM_CALL_E6
#define ST_modKey 	_ROM_CALL_E7
#define ST_precision 	_ROM_CALL_E8
#define ST_readOnly 	_ROM_CALL_E9
#define ST_stack 	_ROM_CALL_EA
#define ST_refDsp 	_ROM_CALL_EB
#define OSCheckBreak 	_ROM_CALL_EC
#define OSClearBreak 	_ROM_CALL_ED
#define OSEnableBreak 	_ROM_CALL_EE
#define OSDisableBreak 	_ROM_CALL_EF
#define OSRegisterTimer 	_ROM_CALL_F0
#define OSFreeTimer 	_ROM_CALL_F1
#define OSTimerCurVal 	_ROM_CALL_F2
#define OSTimerExpired 	_ROM_CALL_F3
#define OSTimerRestart 	_ROM_CALL_F4
#define acos 	_ROM_CALL_F5
#define asin 	_ROM_CALL_F6
#define atan 	_ROM_CALL_F7
#define atan2 	_ROM_CALL_F8
#define cos 	_ROM_CALL_F9
#define sin 	_ROM_CALL_FA
#define tan 	_ROM_CALL_FB
#define cosh 	_ROM_CALL_FC
#define sinh 	_ROM_CALL_FD
#define tanh 	_ROM_CALL_FE
#define exp 	_ROM_CALL_FF
#define log 	_ROM_CALL_100
#define log10 	_ROM_CALL_101
#define modf 	_ROM_CALL_102
#define pow 	_ROM_CALL_103
#define sqrt 	_ROM_CALL_104
#define ceil 	_ROM_CALL_105
#define fabs 	_ROM_CALL_106
#define floor 	_ROM_CALL_107
#define fmod 	_ROM_CALL_108
#define top_estack 	_ROM_CALL_109
#define next_expression_index 	_ROM_CALL_10A
#define gr_active 	_ROM_CALL_10B
#define gr_other 	_ROM_CALL_10C
#define ABT_dialog 	_ROM_CALL_10D
#define HomeExecute 	_ROM_CALL_10E
#define HomePushEStack 	_ROM_CALL_10F
#define SP_Define 	_ROM_CALL_110
#define store_data_var 	_ROM_CALL_111
#define recall_data_var 	_ROM_CALL_112
#define CharNumber 	_ROM_CALL_113
#define spike_optionD 	_ROM_CALL_114
#define spike_geo_titles 	_ROM_CALL_115
#define spike_in_editor 	_ROM_CALL_116
#define dv_create_graph_titles 	_ROM_CALL_117
#define spike_titles_in_editor 	_ROM_CALL_118
#define dv_findColumn 	_ROM_CALL_119
#define spike_chk_gr_dirty 	_ROM_CALL_11A
#define GetStatValue 	_ROM_CALL_11B
#define partial_len 	_ROM_CALL_11C
#define paint_all_except 	_ROM_CALL_11D
#define EQU_select 	_ROM_CALL_11E
#define EQU_setStyle 	_ROM_CALL_11F
#define EQU_getNameInfo 	_ROM_CALL_120
#define checkCurrent 	_ROM_CALL_121
#define BN_power17Mod 	_ROM_CALL_122
#define BN_powerMod 	_ROM_CALL_123
#define BN_prodMod 	_ROM_CALL_124
#define CAT_dialog 	_ROM_CALL_125
#define caddcert 	_ROM_CALL_126
#define cdecrypt 	_ROM_CALL_127
#define ceof 	_ROM_CALL_128
#define cfindcertfield 	_ROM_CALL_129
#define cfindfield 	_ROM_CALL_12A
#define cgetc 	_ROM_CALL_12B
#define cgetcert 	_ROM_CALL_12C
#define cgetflen 	_ROM_CALL_12D
#define cgetfnl 	_ROM_CALL_12E
#define cgetnl 	_ROM_CALL_12F
#define cgetns 	_ROM_CALL_130
#define cgetvernum 	_ROM_CALL_131
#define copen 	_ROM_CALL_132
#define copensub 	_ROM_CALL_133
#define cputhdr 	_ROM_CALL_134
#define cputnl 	_ROM_CALL_135
#define cputns 	_ROM_CALL_136
#define cread 	_ROM_CALL_137
#define ctell 	_ROM_CALL_138
#define cwrite 	_ROM_CALL_139
#define cacos 	_ROM_CALL_13A
#define casin 	_ROM_CALL_13B
#define catan 	_ROM_CALL_13C
#define cacosh 	_ROM_CALL_13D
#define casinh 	_ROM_CALL_13E
#define catanh 	_ROM_CALL_13F
#define ccos 	_ROM_CALL_140
#define csin 	_ROM_CALL_141
#define ctan 	_ROM_CALL_142
#define ccosh 	_ROM_CALL_143
#define csinh 	_ROM_CALL_144
#define ctanh 	_ROM_CALL_145
#define csqrt 	_ROM_CALL_146
#define cln 	_ROM_CALL_147
#define clog10 	_ROM_CALL_148
#define cexp 	_ROM_CALL_149
#define CustomBegin 	_ROM_CALL_14A
#define CustomMenuItem 	_ROM_CALL_14B
#define CustomEnd 	_ROM_CALL_14C
#define ReallocExprStruct 	_ROM_CALL_14D
#define SearchExprStruct 	_ROM_CALL_14E
#define handleRclKey 	_ROM_CALL_14F
#define CustomFree 	_ROM_CALL_150
#define ERD_dialog 	_ROM_CALL_151
#define ERD_process 	_ROM_CALL_152
#define ER_throwVar 	_ROM_CALL_153
#define ER_catch 	_ROM_CALL_154
#define ER_success 	_ROM_CALL_155
#define EV_centralDispatcher 	_ROM_CALL_156
#define EV_defaultHandler 	_ROM_CALL_157
#define EV_eventLoop 	_ROM_CALL_158
#define EV_registerMenu 	_ROM_CALL_159
#define EX_patch 	_ROM_CALL_15A
#define EM_abandon 	_ROM_CALL_15B
#define EM_blockErase 	_ROM_CALL_15C
#define EM_blockVerifyErase 	_ROM_CALL_15D
#define EM_delete 	_ROM_CALL_15E
#define EM_findEmptySlot 	_ROM_CALL_15F
#define EM_GC 	_ROM_CALL_160
#define EM_moveSymFromExtMem 	_ROM_CALL_161
#define EM_moveSymToExtMem 	_ROM_CALL_162
#define EM_open 	_ROM_CALL_163
#define EM_put 	_ROM_CALL_164
#define EM_survey 	_ROM_CALL_165
#define EM_twinSymFromExtMem 	_ROM_CALL_166
#define EM_write 	_ROM_CALL_167
#define EM_writeToExtMem 	_ROM_CALL_168
#define FL_addCert 	_ROM_CALL_169
#define FL_download 	_ROM_CALL_16A
#define FL_getHardwareParmBlock 	_ROM_CALL_16B
#define FL_getCert 	_ROM_CALL_16C
#define FL_getVerNum 	_ROM_CALL_16D
#define EQU_deStatus 	_ROM_CALL_16E
#define cmpstri 	_ROM_CALL_16F
#define fix_loop_displacements 	_ROM_CALL_170
#define FL_write 	_ROM_CALL_171
#define fpisanint 	_ROM_CALL_172
#define fpisodd 	_ROM_CALL_173
#define round12 	_ROM_CALL_174
#define round14 	_ROM_CALL_175
#define GD_Circle 	_ROM_CALL_176
#define GD_Line 	_ROM_CALL_177
#define GD_HVLine 	_ROM_CALL_178
#define GD_Pen 	_ROM_CALL_179
#define GD_Eraser 	_ROM_CALL_17A
#define GD_Text 	_ROM_CALL_17B
#define GD_Select 	_ROM_CALL_17C
#define GD_Contour 	_ROM_CALL_17D
#define GKeyIn 	_ROM_CALL_17E
#define GKeyDown 	_ROM_CALL_17F
#define GKeyFlush 	_ROM_CALL_180
#define HelpKeys 	_ROM_CALL_181
#define QModeKey 	_ROM_CALL_182
#define QSysKey 	_ROM_CALL_183
#define WordInList 	_ROM_CALL_184
#define BitmapGet 	_ROM_CALL_185
#define BitmapInit 	_ROM_CALL_186
#define BitmapPut 	_ROM_CALL_187
#define BitmapSize 	_ROM_CALL_188
#define ScrRectFill 	_ROM_CALL_189
#define ScrRectOverlap 	_ROM_CALL_18A
#define ScrRectScroll 	_ROM_CALL_18B
#define ScrRectShift 	_ROM_CALL_18C
#define QScrRectOverlap 	_ROM_CALL_18D
#define FontGetSys 	_ROM_CALL_18E
#define FontSetSys 	_ROM_CALL_18F
#define FontCharWidth 	_ROM_CALL_190
#define DrawClipChar 	_ROM_CALL_191
#define DrawClipEllipse 	_ROM_CALL_192
#define DrawClipLine 	_ROM_CALL_193
#define DrawClipPix 	_ROM_CALL_194
#define DrawClipRect 	_ROM_CALL_195
#define DrawMultiLines 	_ROM_CALL_196
#define DrawStrWidth 	_ROM_CALL_197
#define FillTriangle 	_ROM_CALL_198
#define FillLines2 	_ROM_CALL_199
#define SetCurAttr 	_ROM_CALL_19A
#define SetCurClip 	_ROM_CALL_19B
#define LineTo 	_ROM_CALL_19C
#define MoveTo 	_ROM_CALL_19D
#define ScreenClear 	_ROM_CALL_19E
#define GetPix 	_ROM_CALL_19F
#define SaveScrState 	_ROM_CALL_1A0
#define RestoreScrState 	_ROM_CALL_1A1
#define PortSet 	_ROM_CALL_1A2
#define PortRestore 	_ROM_CALL_1A3
#define DrawChar 	_ROM_CALL_1A4
#define DrawFkey 	_ROM_CALL_1A5
#define DrawIcon 	_ROM_CALL_1A6
#define DrawLine 	_ROM_CALL_1A7
#define DrawPix 	_ROM_CALL_1A8
#define DrawStr 	_ROM_CALL_1A9
#define GM_Value 	_ROM_CALL_1AA
#define GM_Intersect 	_ROM_CALL_1AB
#define GM_Integrate 	_ROM_CALL_1AC
#define GM_Inflection 	_ROM_CALL_1AD
#define GM_TanLine 	_ROM_CALL_1AE
#define GM_Math1 	_ROM_CALL_1AF
#define GM_Derivative 	_ROM_CALL_1B0
#define GM_DistArc 	_ROM_CALL_1B1
#define GM_Shade 	_ROM_CALL_1B2
#define YCvtFtoWin 	_ROM_CALL_1B3
#define DlgMessage 	_ROM_CALL_1B4
#define SetGraphMode 	_ROM_CALL_1B5
#define Regraph 	_ROM_CALL_1B6
#define GrAxes 	_ROM_CALL_1B7
#define gr_xres_pixel 	_ROM_CALL_1B8
#define CptFuncX 	_ROM_CALL_1B9
#define XCvtPtoF 	_ROM_CALL_1BA
#define YCvtPtoF 	_ROM_CALL_1BB
#define YCvtFtoP 	_ROM_CALL_1BC
#define XCvtFtoP 	_ROM_CALL_1BD
#define GrLineFlt 	_ROM_CALL_1BE
#define FuncLineFlt 	_ROM_CALL_1BF
#define GrClipLine 	_ROM_CALL_1C0
#define CptDeltax 	_ROM_CALL_1C1
#define CptDeltay 	_ROM_CALL_1C2
#define CkValidDelta 	_ROM_CALL_1C3
#define GR_Pan 	_ROM_CALL_1C4
#define FindFunc 	_ROM_CALL_1C5
#define FindGrFunc 	_ROM_CALL_1C6
#define grFuncName 	_ROM_CALL_1C7
#define gr_initCondName 	_ROM_CALL_1C8
#define CptIndep 	_ROM_CALL_1C9
#define gr_CptIndepInc 	_ROM_CALL_1CA
#define gr_del_locals 	_ROM_CALL_1CB
#define gr_DelFolder 	_ROM_CALL_1CC
#define gr_openFolder 	_ROM_CALL_1CD
#define setup_more_graph_fun 	_ROM_CALL_1CE
#define unlock_more_graph_fun 	_ROM_CALL_1CF
#define execute_graph_func 	_ROM_CALL_1D0
#define cpt_gr_fun 	_ROM_CALL_1D1
#define cpt_gr_param 	_ROM_CALL_1D2
#define cpt_gr_polar 	_ROM_CALL_1D3
#define gr_execute_seq 	_ROM_CALL_1D4
#define CountGrFunc 	_ROM_CALL_1D5
#define FirstSeqPlot 	_ROM_CALL_1D6
#define cleanup_seq_mem 	_ROM_CALL_1D7
#define time_loop 	_ROM_CALL_1D8
#define InitTimeSeq 	_ROM_CALL_1D9
#define seqWebInit 	_ROM_CALL_1DA
#define run_one_seq 	_ROM_CALL_1DB
#define gr_seq_value 	_ROM_CALL_1DC
#define StepCk 	_ROM_CALL_1DD
#define seqStepCk 	_ROM_CALL_1DE
#define rngLen 	_ROM_CALL_1DF
#define gdb_len 	_ROM_CALL_1E0
#define gdb_store 	_ROM_CALL_1E1
#define gdb_recall 	_ROM_CALL_1E2
#define gr_DispLabels 	_ROM_CALL_1E3
#define GraphOrTableCmd 	_ROM_CALL_1E4
#define ck_valid_float 	_ROM_CALL_1E5
#define CreateEmptyList 	_ROM_CALL_1E6
#define QSkipGraphErr 	_ROM_CALL_1E7
#define gr_find_de_result 	_ROM_CALL_1E8
#define InitDEAxesRng 	_ROM_CALL_1E9
#define InitDEMem 	_ROM_CALL_1EA
#define de_loop 	_ROM_CALL_1EB
#define cleanup_de_mem 	_ROM_CALL_1EC
#define gr_de_value 	_ROM_CALL_1ED
#define gr_find_func_index 	_ROM_CALL_1EE
#define CptLastIndepDE 	_ROM_CALL_1EF
#define de_initRes 	_ROM_CALL_1F0
#define gr_del_vars_in_folder 	_ROM_CALL_1F1
#define gr_de_axes_lbl 	_ROM_CALL_1F2
#define gr_execute_de 	_ROM_CALL_1F3
#define gr_delete_fldpic 	_ROM_CALL_1F4
#define gr_remove_fldpic 	_ROM_CALL_1F5
#define gr_add_fldpic 	_ROM_CALL_1F6
#define gr_stopic 	_ROM_CALL_1F7
#define gr_find_el 	_ROM_CALL_1F8
#define deStepCk 	_ROM_CALL_1F9
#define gr_ck_solvergraph 	_ROM_CALL_1FA
#define GR3_addContours 	_ROM_CALL_1FB
#define GraphActivate 	_ROM_CALL_1FC
#define GR3_freeDB 	_ROM_CALL_1FD
#define GR3_handleEvent 	_ROM_CALL_1FE
#define GR3_paint3d 	_ROM_CALL_1FF
#define GR3_xyToWindow 	_ROM_CALL_200
#define GS_PlotTrace 	_ROM_CALL_201
#define GS_PlotAll 	_ROM_CALL_202
#define PlotDel 	_ROM_CALL_203
#define PlotPut 	_ROM_CALL_204
#define PlotGet 	_ROM_CALL_205
#define PlotInit 	_ROM_CALL_206
#define PlotDup 	_ROM_CALL_207
#define PlotSize 	_ROM_CALL_208
#define PlotLookup 	_ROM_CALL_209
#define QActivePlots 	_ROM_CALL_20A
#define QPlotActive 	_ROM_CALL_20B
#define GT_BackupToScr 	_ROM_CALL_20C
#define GT_CalcDepVals 	_ROM_CALL_20D
#define GT_CenterGraphCursor 	_ROM_CALL_20E
#define GT_CursorKey 	_ROM_CALL_20F
#define GT_DspFreeTraceCoords 	_ROM_CALL_210
#define GT_DspTraceCoords 	_ROM_CALL_211
#define GT_DspMsg 	_ROM_CALL_212
#define GT_Error 	_ROM_CALL_213
#define GT_Format 	_ROM_CALL_214
#define GT_FreeTrace 	_ROM_CALL_215
#define GT_IncXY 	_ROM_CALL_216
#define GT_KeyIn 	_ROM_CALL_217
#define GT_QFloatCursorsInRange 	_ROM_CALL_218
#define GT_Regraph 	_ROM_CALL_219
#define GT_Regraph_if_neccy 	_ROM_CALL_21A
#define GT_Open 	_ROM_CALL_21B
#define GT_SaveAs 	_ROM_CALL_21C
#define GT_SelFunc 	_ROM_CALL_21D
#define GT_SetGraphRange 	_ROM_CALL_21E
#define GT_SetCursorXY 	_ROM_CALL_21F
#define GT_ShowMarkers 	_ROM_CALL_220
#define GT_Trace 	_ROM_CALL_221
#define GT_ValidGraphRanges 	_ROM_CALL_222
#define GT_WinBound 	_ROM_CALL_223
#define GT_WinCursor 	_ROM_CALL_224
#define GYcoord 	_ROM_CALL_225
#define GXcoord 	_ROM_CALL_226
#define round12_err 	_ROM_CALL_227
#define GT_Set_Graph_Format 	_ROM_CALL_228
#define GT_PrintCursor 	_ROM_CALL_229
#define GT_DE_Init_Conds 	_ROM_CALL_22A
#define GZ_Box 	_ROM_CALL_22B
#define GZ_Center 	_ROM_CALL_22C
#define GZ_Decimal 	_ROM_CALL_22D
#define GZ_Fit 	_ROM_CALL_22E
#define GZ_InOut 	_ROM_CALL_22F
#define GZ_Integer 	_ROM_CALL_230
#define GZ_Previous 	_ROM_CALL_231
#define GZ_Recall 	_ROM_CALL_232
#define GZ_SetFactors 	_ROM_CALL_233
#define GZ_Square 	_ROM_CALL_234
#define GZ_Standard 	_ROM_CALL_235
#define GZ_Stat 	_ROM_CALL_236
#define GZ_Store 	_ROM_CALL_237
#define GZ_Trig 	_ROM_CALL_238
#define HeapGetHandle 	_ROM_CALL_239
#define HeapPtrToHandle 	_ROM_CALL_23A
#define FreeHandles 	_ROM_CALL_23B
#define HS_chopFIFO 	_ROM_CALL_23C
#define HS_countFIFO 	_ROM_CALL_23D
#define HS_deleteFIFONode 	_ROM_CALL_23E
#define HS_freeAll 	_ROM_CALL_23F
#define HS_freeFIFONode 	_ROM_CALL_240
#define HS_getAns 	_ROM_CALL_241
#define HS_getEntry 	_ROM_CALL_242
#define HS_getFIFONode 	_ROM_CALL_243
#define HS_popEStack 	_ROM_CALL_244
#define HS_newFIFONode 	_ROM_CALL_245
#define HS_pushFIFONode 	_ROM_CALL_246
#define HToESI 	_ROM_CALL_247
#define OSInitKeyInitDelay 	_ROM_CALL_248
#define OSInitBetweenKeyDelay 	_ROM_CALL_249
#define OSCheckSilentLink 	_ROM_CALL_24A
#define OSLinkCmd 	_ROM_CALL_24B
#define OSLinkReset 	_ROM_CALL_24C
#define OSLinkOpen 	_ROM_CALL_24D
#define OSLinkClose 	_ROM_CALL_24E
#define OSReadLinkBlock 	_ROM_CALL_24F
#define OSWriteLinkBlock 	_ROM_CALL_250
#define OSLinkTxQueueInquire 	_ROM_CALL_251
#define OSLinkTxQueueActive 	_ROM_CALL_252
#define LIO_SendProduct 	_ROM_CALL_253
#define MD5Init 	_ROM_CALL_254
#define MD5Update 	_ROM_CALL_255
#define MD5Final 	_ROM_CALL_256
#define MD5Done 	_ROM_CALL_257
#define convert_to_TI_92 	_ROM_CALL_258
#define gen_version 	_ROM_CALL_259
#define is_executable 	_ROM_CALL_25A
#define NG_RPNToText 	_ROM_CALL_25B
#define NG_approxESI 	_ROM_CALL_25C
#define NG_execute 	_ROM_CALL_25D
#define NG_graphESI 	_ROM_CALL_25E
#define NG_rationalESI 	_ROM_CALL_25F
#define NG_tokenize 	_ROM_CALL_260
#define NG_setup_graph_fun 	_ROM_CALL_261
#define NG_cleanup_graph_fun 	_ROM_CALL_262
#define push_END_TAG 	_ROM_CALL_263
#define push_LIST_TAG 	_ROM_CALL_264
#define tokenize_if_TI_92_or_text 	_ROM_CALL_265
#define setjmp 	_ROM_CALL_266
#define longjmp 	_ROM_CALL_267
#define VarGraphRefBitsClear 	_ROM_CALL_268
#define VarInit 	_ROM_CALL_269
#define memcpy 	_ROM_CALL_26A
#define memmove 	_ROM_CALL_26B
#define strcpy 	_ROM_CALL_26C
#define strncpy 	_ROM_CALL_26D
#define strcat 	_ROM_CALL_26E
#define strncat 	_ROM_CALL_26F
#define memcmp 	_ROM_CALL_270
#define strcmp 	_ROM_CALL_271
#define strncmp 	_ROM_CALL_272
#define memchr 	_ROM_CALL_273
#define strchr 	_ROM_CALL_274
#define strcspn 	_ROM_CALL_275
#define strpbrk 	_ROM_CALL_276
#define strrchr 	_ROM_CALL_277
#define strspn 	_ROM_CALL_278
#define strstr 	_ROM_CALL_279
#define strtok 	_ROM_CALL_27A
#define _memset 	_ROM_CALL_27B
#define memset 	_ROM_CALL_27C
#define strerror 	_ROM_CALL_27D
#define strlen 	_ROM_CALL_27E
#define SymAddTwin 	_ROM_CALL_27F
#define SymDelTwin 	_ROM_CALL_280
#define LoadSymFromFindHandle 	_ROM_CALL_281
#define MakeHsym 	_ROM_CALL_282
#define SymFindPtr 	_ROM_CALL_283
#define OSVRegisterTimer 	_ROM_CALL_284
#define OSVFreeTimer 	_ROM_CALL_285
#define sincos 	_ROM_CALL_286
#define asinh 	_ROM_CALL_287
#define acosh 	_ROM_CALL_288
#define atanh 	_ROM_CALL_289
#define itrig 	_ROM_CALL_28A
#define trig 	_ROM_CALL_28B
#define VarOpen 	_ROM_CALL_28C
#define VarSaveAs 	_ROM_CALL_28D
#define VarNew 	_ROM_CALL_28E
#define VarCreateFolderPopup 	_ROM_CALL_28F
#define VarSaveTitle 	_ROM_CALL_290
#define WinWidth 	_ROM_CALL_291
#define WinHeight 	_ROM_CALL_292
#define XR_stringPtr 	_ROM_CALL_293
#define OSReset 	_ROM_CALL_294
#define SumStoChkMem 	_ROM_CALL_295
#define OSContrastUp 	_ROM_CALL_296
#define OSContrastDn 	_ROM_CALL_297
#define OSKeyScan 	_ROM_CALL_298
#define OSGetStatKeys 	_ROM_CALL_299
#define off 	_ROM_CALL_29A
#define idle 	_ROM_CALL_29B
#define OSSetSR 	_ROM_CALL_29C
#define AB_prodid 	_ROM_CALL_29D
#define AB_prodname 	_ROM_CALL_29E
#define AB_serno 	_ROM_CALL_29F
#define cgetcertrevno 	_ROM_CALL_2A0
#define cgetsn 	_ROM_CALL_2A1
#define de_rng_no_graph 	_ROM_CALL_2A2
#define EV_hook 	_ROM_CALL_2A3
#define _ds16u16 	_ROM_CALL_2A4
#define _ms16u16 	_ROM_CALL_2A5
#define _du16u16 	_ROM_CALL_2A6
#define _mu16u16 	_ROM_CALL_2A7
#define _ds32s32 	_ROM_CALL_2A8
#define _ms32s32 	_ROM_CALL_2A9
#define _du32u32 	_ROM_CALL_2AA
#define _mu32u32 	_ROM_CALL_2AB
#define assign_between 	_ROM_CALL_2AC
#define did_push_var_val 	_ROM_CALL_2AD
#define does_push_fetch 	_ROM_CALL_2AE
#define delete_list_element 	_ROM_CALL_2AF
#define push_ans_entry 	_ROM_CALL_2B0
#define index_after_match_endtag 	_ROM_CALL_2B1
#define push_indir_name 	_ROM_CALL_2B2
#define push_user_func 	_ROM_CALL_2B3
#define store_func_def 	_ROM_CALL_2B4
#define store_to_subscripted_element 	_ROM_CALL_2B5
#define index_below_display_expression_aux 	_ROM_CALL_2B6
#define get_key_ptr 	_ROM_CALL_2B7
#define get_list_indices 	_ROM_CALL_2B8
#define get_matrix_indices 	_ROM_CALL_2B9
#define init_list_indices 	_ROM_CALL_2BA
#define init_matrix_indices 	_ROM_CALL_2BB
#define push_float_qr_fact 	_ROM_CALL_2BC
#define push_lu_fact 	_ROM_CALL_2BD
#define push_symbolic_qr_fact 	_ROM_CALL_2BE
#define are_expressions_identical 	_ROM_CALL_2BF
#define compare_expressions 	_ROM_CALL_2C0
#define find_error_message 	_ROM_CALL_2C1
#define check_estack_size 	_ROM_CALL_2C2
#define delete_between 	_ROM_CALL_2C3
#define deleted_between 	_ROM_CALL_2C4
#define delete_expression 	_ROM_CALL_2C5
#define deleted_expression 	_ROM_CALL_2C6
#define estack_to_short 	_ROM_CALL_2C7
#define estack_to_ushort 	_ROM_CALL_2C8
#define factor_base_index 	_ROM_CALL_2C9
#define factor_exponent_index 	_ROM_CALL_2CA
#define GetValue 	_ROM_CALL_2CB
#define im_index 	_ROM_CALL_2CC
#define index_numeric_term 	_ROM_CALL_2CD
#define index_of_lead_base_of_lead_term 	_ROM_CALL_2CE
#define index_main_var 	_ROM_CALL_2CF
#define is_advanced_tag 	_ROM_CALL_2D0
#define is_antisymmetric 	_ROM_CALL_2D1
#define is_complex_number 	_ROM_CALL_2D2
#define is_complex0 	_ROM_CALL_2D3
#define is_free_of_tag 	_ROM_CALL_2D4
#define is_independent_of 	_ROM_CALL_2D5
#define is_independent_of_de_seq_vars 	_ROM_CALL_2D6
#define is_independent_of_tail 	_ROM_CALL_2D7
#define is_independent_of_elements 	_ROM_CALL_2D8
#define is_monomial 	_ROM_CALL_2D9
#define is_monomial_in_kernel 	_ROM_CALL_2DA
#define is_narrowly_independent_of 	_ROM_CALL_2DB
#define is_symmetric 	_ROM_CALL_2DC
#define is_tail_independent_of 	_ROM_CALL_2DD
#define lead_base_index 	_ROM_CALL_2DE
#define lead_exponent_index 	_ROM_CALL_2DF
#define lead_factor_index 	_ROM_CALL_2E0
#define lead_term_index 	_ROM_CALL_2E1
#define main_gen_var_index 	_ROM_CALL_2E2
#define map_unary_over_comparison 	_ROM_CALL_2E3
#define min_quantum 	_ROM_CALL_2E4
#define move_between_to_top 	_ROM_CALL_2E5
#define moved_between_to_top 	_ROM_CALL_2E6
#define numeric_factor_index 	_ROM_CALL_2E7
#define push_between 	_ROM_CALL_2E8
#define push_expr_quantum 	_ROM_CALL_2E9
#define push_expr2_quantum 	_ROM_CALL_2EA
#define push_next_arb_int 	_ROM_CALL_2EB
#define push_next_arb_real 	_ROM_CALL_2EC
#define push_next_internal_var 	_ROM_CALL_2ED
#define push_quantum 	_ROM_CALL_2EE
#define push_quantum_pair 	_ROM_CALL_2EF
#define reductum_index 	_ROM_CALL_2F0
#define remaining_factors_index 	_ROM_CALL_2F1
#define re_index 	_ROM_CALL_2F2
#define reset_estack_size 	_ROM_CALL_2F3
#define reset_control_flags 	_ROM_CALL_2F4
#define can_be_approxed 	_ROM_CALL_2F5
#define compare_complex_magnitudes 	_ROM_CALL_2F6
#define compare_Floats 	_ROM_CALL_2F7
#define did_push_cnvrt_Float_to_integer 	_ROM_CALL_2F8
#define estack_number_to_Float 	_ROM_CALL_2F9
#define float_class 	_ROM_CALL_2FA
#define frexp10 	_ROM_CALL_2FB
#define gcd_exact_whole_Floats 	_ROM_CALL_2FC
#define init_float 	_ROM_CALL_2FD
#define is_Float_exact_whole_number 	_ROM_CALL_2FE
#define is_float_infinity 	_ROM_CALL_2FF
#define is_float_negative_zero 	_ROM_CALL_300
#define is_float_positive_zero 	_ROM_CALL_301
#define is_float_signed_infinity 	_ROM_CALL_302
#define is_float_transfinite 	_ROM_CALL_303
#define is_float_unsigned_inf_or_nan 	_ROM_CALL_304
#define is_float_unsigned_zero 	_ROM_CALL_305
#define is_nan 	_ROM_CALL_306
#define likely_approx_to_complex_number 	_ROM_CALL_307
#define likely_approx_to_number 	_ROM_CALL_308
#define norm1_complex_Float 	_ROM_CALL_309
#define push_Float 	_ROM_CALL_30A
#define push_Float_to_nonneg_int 	_ROM_CALL_30B
#define push_Float_to_rat 	_ROM_CALL_30C
#define push_cnvrt_integer_if_whole_nmb 	_ROM_CALL_30D
#define push_overflow_to_infinity 	_ROM_CALL_30E
#define push_pow 	_ROM_CALL_30F
#define push_round_Float 	_ROM_CALL_310
#define should_and_did_push_approx_arg2 	_ROM_CALL_311
#define signum_Float 	_ROM_CALL_312
#define did_push_to_polar 	_ROM_CALL_313
#define push_degrees 	_ROM_CALL_314
#define push_format 	_ROM_CALL_315
#define push_getkey 	_ROM_CALL_316
#define push_getfold 	_ROM_CALL_317
#define push_getmode 	_ROM_CALL_318
#define push_gettype 	_ROM_CALL_319
#define push_instring 	_ROM_CALL_31A
#define push_mrow_aux 	_ROM_CALL_31B
#define push_part 	_ROM_CALL_31C
#define push_pttest 	_ROM_CALL_31D
#define push_pxltest 	_ROM_CALL_31E
#define push_rand 	_ROM_CALL_31F
#define push_randpoly 	_ROM_CALL_320
#define push_setfold 	_ROM_CALL_321
#define push_setgraph 	_ROM_CALL_322
#define push_setmode 	_ROM_CALL_323
#define push_settable 	_ROM_CALL_324
#define push_str_to_expr 	_ROM_CALL_325
#define push_string 	_ROM_CALL_326
#define push_switch 	_ROM_CALL_327
#define push_to_cylin 	_ROM_CALL_328
#define push_to_sphere 	_ROM_CALL_329
#define cmd_andpic 	_ROM_CALL_32A
#define cmd_blddata 	_ROM_CALL_32B
#define cmd_circle 	_ROM_CALL_32C
#define cmd_clrdraw 	_ROM_CALL_32D
#define cmd_clrerr 	_ROM_CALL_32E
#define cmd_clrgraph 	_ROM_CALL_32F
#define cmd_clrhome 	_ROM_CALL_330
#define cmd_clrio 	_ROM_CALL_331
#define cmd_clrtable 	_ROM_CALL_332
#define cmd_copyvar 	_ROM_CALL_333
#define cmd_cubicreg 	_ROM_CALL_334
#define cmd_custmoff 	_ROM_CALL_335
#define cmd_custmon 	_ROM_CALL_336
#define cmd_custom 	_ROM_CALL_337
#define cmd_cycle 	_ROM_CALL_338
#define cmd_cyclepic 	_ROM_CALL_339
#define cmd_delfold 	_ROM_CALL_33A
#define cmd_delvar 	_ROM_CALL_33B
#define cmd_dialog 	_ROM_CALL_33C
#define cmd_disp 	_ROM_CALL_33D
#define cmd_dispg 	_ROM_CALL_33E
#define cmd_disphome 	_ROM_CALL_33F
#define cmd_disptbl 	_ROM_CALL_340
#define cmd_drawfunc 	_ROM_CALL_341
#define cmd_drawinv 	_ROM_CALL_342
#define cmd_drawparm 	_ROM_CALL_343
#define cmd_drawpol 	_ROM_CALL_344
#define cmd_else 	_ROM_CALL_345
#define cmd_endfor 	_ROM_CALL_346
#define cmd_endloop 	_ROM_CALL_347
#define cmd_endtry 	_ROM_CALL_348
#define cmd_endwhile 	_ROM_CALL_349
#define cmd_exit 	_ROM_CALL_34A
#define cmd_expreg 	_ROM_CALL_34B
#define cmd_fill 	_ROM_CALL_34C
#define cmd_fnoff 	_ROM_CALL_34D
#define cmd_fnon 	_ROM_CALL_34E
#define cmd_for 	_ROM_CALL_34F
#define cmd_get 	_ROM_CALL_350
#define cmd_getcalc 	_ROM_CALL_351
#define cmd_goto 	_ROM_CALL_352
#define cmd_graph 	_ROM_CALL_353
#define cmd_if 	_ROM_CALL_354
#define cmd_ifthen 	_ROM_CALL_355
#define cmd_input 	_ROM_CALL_356
#define cmd_inputstr 	_ROM_CALL_357
#define cmd_line 	_ROM_CALL_358
#define cmd_linehorz 	_ROM_CALL_359
#define cmd_linetan 	_ROM_CALL_35A
#define cmd_linevert 	_ROM_CALL_35B
#define cmd_linreg 	_ROM_CALL_35C
#define cmd_lnreg 	_ROM_CALL_35D
#define cmd_local 	_ROM_CALL_35E
#define cmd_lock 	_ROM_CALL_35F
#define cmd_logistic 	_ROM_CALL_360
#define cmd_medmed 	_ROM_CALL_361
#define cmd_movevar 	_ROM_CALL_362
#define cmd_newdata 	_ROM_CALL_363
#define cmd_newfold 	_ROM_CALL_364
#define cmd_newpic 	_ROM_CALL_365
#define cmd_newplot 	_ROM_CALL_366
#define cmd_newprob 	_ROM_CALL_367
#define cmd_onevar 	_ROM_CALL_368
#define cmd_output 	_ROM_CALL_369
#define cmd_passerr 	_ROM_CALL_36A
#define cmd_pause 	_ROM_CALL_36B
#define cmd_plotsoff 	_ROM_CALL_36C
#define cmd_plotson 	_ROM_CALL_36D
#define cmd_popup 	_ROM_CALL_36E
#define cmd_powerreg 	_ROM_CALL_36F
#define cmd_printobj 	_ROM_CALL_370
#define cmd_prompt 	_ROM_CALL_371
#define cmd_ptchg 	_ROM_CALL_372
#define cmd_ptoff 	_ROM_CALL_373
#define cmd_pton 	_ROM_CALL_374
#define cmd_pttext 	_ROM_CALL_375
#define cmd_pxlchg 	_ROM_CALL_376
#define cmd_pxlcircle 	_ROM_CALL_377
#define cmd_pxlhorz 	_ROM_CALL_378
#define cmd_pxlline 	_ROM_CALL_379
#define cmd_pxloff 	_ROM_CALL_37A
#define cmd_pxlon 	_ROM_CALL_37B
#define cmd_pxltext 	_ROM_CALL_37C
#define cmd_pxlvert 	_ROM_CALL_37D
#define cmd_quadreg 	_ROM_CALL_37E
#define cmd_quartreg 	_ROM_CALL_37F
#define cmd_randseed 	_ROM_CALL_380
#define cmd_rclgdb 	_ROM_CALL_381
#define cmd_rclpic 	_ROM_CALL_382
#define cmd_rename 	_ROM_CALL_383
#define cmd_request 	_ROM_CALL_384
#define cmd_return 	_ROM_CALL_385
#define cmd_rplcpic 	_ROM_CALL_386
#define cmd_send 	_ROM_CALL_387
#define cmd_sendcalc 	_ROM_CALL_388
#define cmd_sendchat 	_ROM_CALL_389
#define cmd_shade 	_ROM_CALL_38A
#define cmd_showstat 	_ROM_CALL_38B
#define cmd_sinreg 	_ROM_CALL_38C
#define cmd_slpline 	_ROM_CALL_38D
#define cmd_sorta 	_ROM_CALL_38E
#define cmd_sortd 	_ROM_CALL_38F
#define cmd_stogdb 	_ROM_CALL_390
#define cmd_stopic 	_ROM_CALL_391
#define cmd_style 	_ROM_CALL_392
#define cmd_table 	_ROM_CALL_393
#define cmd_text 	_ROM_CALL_394
#define cmd_toolbar 	_ROM_CALL_395
#define cmd_trace 	_ROM_CALL_396
#define cmd_try 	_ROM_CALL_397
#define cmd_twovar 	_ROM_CALL_398
#define cmd_unlock 	_ROM_CALL_399
#define cmd_while 	_ROM_CALL_39A
#define cmd_xorpic 	_ROM_CALL_39B
#define cmd_zoombox 	_ROM_CALL_39C
#define cmd_zoomdata 	_ROM_CALL_39D
#define cmd_zoomdec 	_ROM_CALL_39E
#define cmd_zoomfit 	_ROM_CALL_39F
#define cmd_zoomin 	_ROM_CALL_3A0
#define cmd_zoomint 	_ROM_CALL_3A1
#define cmd_zoomout 	_ROM_CALL_3A2
#define cmd_zoomprev 	_ROM_CALL_3A3
#define cmd_zoomrcl 	_ROM_CALL_3A4
#define cmd_zoomsqr 	_ROM_CALL_3A5
#define cmd_zoomstd 	_ROM_CALL_3A6
#define cmd_zoomsto 	_ROM_CALL_3A7
#define cmd_zoomtrig 	_ROM_CALL_3A8
#define OSenqueue 	_ROM_CALL_3A9
#define OSdequeue 	_ROM_CALL_3AA
#define OSqinquire 	_ROM_CALL_3AB
#define OSqhead 	_ROM_CALL_3AC
#define OSqclear 	_ROM_CALL_3AD
#define did_push_divide_units 	_ROM_CALL_3AE
#define has_unit_base 	_ROM_CALL_3AF
#define init_unit_system 	_ROM_CALL_3B0
#define is_units_term 	_ROM_CALL_3B1
#define push_auto_units_conversion 	_ROM_CALL_3B2
#define push_unit_system_list 	_ROM_CALL_3B3
#define setup_unit_system 	_ROM_CALL_3B4
#define all_tail 	_ROM_CALL_3B5
#define any_tail 	_ROM_CALL_3B6
#define is_matrix 	_ROM_CALL_3B7
#define is_square_matrix 	_ROM_CALL_3B8
#define is_valid_smap_aggregate 	_ROM_CALL_3B9
#define last_element_index 	_ROM_CALL_3BA
#define map_tail 	_ROM_CALL_3BB
#define map_tail_Int 	_ROM_CALL_3BC
#define push_list_plus 	_ROM_CALL_3BD
#define push_list_times 	_ROM_CALL_3BE
#define push_reversed_tail 	_ROM_CALL_3BF
#define push_sq_matrix_to_whole_number 	_ROM_CALL_3C0
#define push_transpose_aux 	_ROM_CALL_3C1
#define push_zero_partial_column 	_ROM_CALL_3C2
#define remaining_element_count 	_ROM_CALL_3C3
#define push_offset_array 	_ROM_CALL_3C4
#define push_matrix_product 	_ROM_CALL_3C5
#define is_pathname 	_ROM_CALL_3C6
#define next_token 	_ROM_CALL_3C7
#define nonblank 	_ROM_CALL_3C8
#define push_parse_prgm_or_func_text 	_ROM_CALL_3C9
#define push_parse_text 	_ROM_CALL_3CA
#define push_var 	_ROM_CALL_3CB


void ValidateWin( WINDOW *w );

WIN_COORDS WinWidth( WINDOW *w );
WIN_COORDS WinHeight( WINDOW *w );

/*==============================================================================*/


void WinActivate( WINDOW *w );
BYTE WinAttr( WINDOW *w, BYTE Attr );
void WinBackupToScr( WINDOW * w);
void WinBackground( WINDOW *w, BYTE Attr );
void WinBegin( WINDOW *w );
BOOL WinBitmapGet( WINDOW *w, const WIN_RECT *WinRect, BITMAP *Bitmap );
void WinBitmapPut( WINDOW *w, SWORD x0, SWORD y0, BITMAP *Bitmap, WORD Attr);
WORD WinBitmapSize( WINDOW *w, const WIN_RECT WinRect );
void WinChar( WINDOW *w, char c );
void WinCharXY(WINDOW *w, WIN_COORDS x, WIN_COORDS y, char c, short Count );
void WinClose( WINDOW *w );
void WinClr( WINDOW *w );
void WinDeactivate( WINDOW *w );
void WinDupStat( WINDOW *w, BOOL Stat );
void WinEllipse( WINDOW *w, WIN_COORDS x0, WIN_COORDS y0, WIN_COORDS a0, WIN_COORDS b0 );

// void WinEnd( WINDOW *w );
void WinFill( WINDOW *w, const WIN_RECT *r, BYTE Attr );
void WinFillLines2( WINDOW *w, const WIN_RECT *l0, const WIN_RECT *l1, BYTE Attr );
void WinFillTriangle( WINDOW *w, WIN_COORDS ax, WIN_COORDS ay, WIN_COORDS bx,
  WIN_COORDS by, WIN_COORDS cx, WIN_COORDS cy, BYTE Attr );
void WinFont( WINDOW *w, BYTE Font );
void WinGetCursor(WINDOW *w,WIN_COORDS*x,WIN_COORDS*y);
WIN_COORDS WinHeight( WINDOW *w );
void WinHide( WINDOW *w );
void WinHome( WINDOW *w );
void WinIcon( WINDOW *w, WIN_COORDS x0, WIN_COORDS y0, const WORD *Icon );
void WinLine( WINDOW *w, const WIN_RECT *Line );
void WinLineNC( WINDOW *w, const WIN_RECT *Line );
void WinLineRel( WINDOW *w, WIN_COORDS x0, WIN_COORDS y0 );
void WinLineTo( WINDOW *w, WIN_COORDS x0, WIN_COORDS y0 );
void WinMoveCursor( WINDOW *w, WIN_COORDS x, WIN_COORDS y );
void WinMoveRel( WINDOW *w, WIN_COORDS x0, WIN_COORDS y0 );
void WinMoveTo( WINDOW *w, WIN_COORDS x0, WIN_COORDS y0 );
BOOL WinOpen( WINDOW *w, const WIN_RECT *r, WORD Flags, ... );
WORD WinPixGet( WINDOW *w, WIN_COORDS x0, WIN_COORDS y0 );
void WinPixSet( WINDOW *w, WIN_COORDS x0, WIN_COORDS y0 );
void WinRect( WINDOW *w, const WIN_RECT *r, BYTE BoxAttr );
BOOL WinReOpen( WINDOW *w, const WIN_RECT *wr, WORD Flags, ... );

void WinScrollH( WINDOW *w, const WIN_RECT *r, SWORD NumCols );
void WinScrollV( WINDOW *w, const WIN_RECT *r, SWORD NumRows );
void WinStr( WINDOW *w, char *Str );
void WinStrXY( WINDOW *w, WIN_COORDS x, WIN_COORDS y, char *Str );


WIN_COORDS WinWidth( WINDOW *w );



/*==============================================================================*/

short FreeHandles( void );
BYTE *HeapAllocPtr( DWORD Hlen );
void HeapFreePtr( BYTE *adr );
HANDLE HeapAlloc( DWORD Hlen );
HANDLE HeapAllocHigh( DWORD Hlen );
HANDLE HeapAllocHighThrow( DWORD Hlen );
HANDLE HeapAllocThrow( DWORD Hlen );
DWORD HeapAvail( void );
void HeapCompress( void );
void HeapFree( HANDLE Handle );
void HeapFreeIndir( HANDLE *PtrHandle );
BOOL   HeapGetLock( HANDLE handle );
HANDLE HeapLock( HANDLE Handle );
DWORD HeapMax( void );
HANDLE HeapMoveHigh( HANDLE h );
HANDLE HeapRealloc( HANDLE Handle, WORD NewHsize );
WORD HeapSize( HANDLE Handle );
HANDLE HeapUnlock( HANDLE Handle );
void * Hlock( HANDLE h );
void * HeapDeref(HANDLE h);


void EM_survey(long *used,long *was_used,long *free,long *unknown1,long *unknown2);
void ST_modKey(short flags);

extern ESI top_estack;
ESI Parse2DExpr(ESI the_object,short flag);
ESI HToESI(short handle);
void Parms2D(ESI display_data,short *width,short *top,short *bottom);
ESI Print2DExpr(ESI display_data,WINDOW *window,short x,short y);

void ER_throwVar(WORD errorNum);
WORD ER_catch(void *ErrorFrame);
void ER_success(void);
void ERD_dialog(WORD errorNum);
void ST_busy(WORD mode);
void ST_eraseHelp(void);
void ST_showHelp(BYTE *message);

short push_parse_text(char*);
void push_user_func(ESI, short);
void push_quantum(unsigned char);
void * HLock(HANDLE);
void NG_execute(HANDLE, short);

short GKeyIn(SCR_RECT *cursor_shape,short flags);
BOOL GKeyDown(void);
void GKeyFlush(void);

void *memcpy(void *, const void *, ULONG);
void *memmove(void *dest, const void *source, ULONG num);
char *strcpy(char *s1, const char *s2);
char *strncpy(char *s1, const char *s2, size_t max);
char *strcat(char *s1, const char *s2);
char *strncat(char *s1, const char *s2, size_t max);
long memcmp(const void *s1, const void *s2, size_t num);
long strcmp(const char *s1, const char *s2);
long strncmp(const char *s1, const char *s2, size_t max);
void *memchr(const void *string, ULONG ch, size_t num);
size_t strlen(const char *string);
char *strchr(const char *s, int c);
size_t strcspn(const char *s1, const char *set);
char *strpbrk(const char *s1, const char *set);
char *strrchr(const char *s1, int c);
size_t strspn(const char *s1, const char *set);
char *strstr(const char *s1, const char *s2);
char *strtok(char *s1, const char *s2);
void *memset(void *buffer, ULONG ch, size_t num);
char *strerror(int error);

void DrawStr(short, short, char*,short);
int DrawStrWidth(char *str, BYTE font);

void PortSet(BYTE *DUP_SCR, BYTE x2, BYTE y2);
void PortRestore(void);





// Adds the sym or frees the handle and returns the HSym, 0 if pb
HSym SymAdd(char *Name);

// Adds the sym in main or frees the handle and returns the HSym, 0 if pb
HSym SymAddMain(char *Name);

// Dels the sym and returns non-0 if ok
int SymDel(char *Name);

// Dels the sym and returns non-0 if ok
int HSymDel(HSym Sym);

// Finds the sym and returns the HSym, 0 if pb
HSym SymFind(char *Name);

// Finds the sym in main and returns the HSym, 0 if pb
HSym SymFindMain(char *Name);

// Moves the sym and returns non-0 if ok
int SymMove(char *SrcName, char *DestName);

// Derefs a HSym and returns a pointer to the folder entry, NULL if pb
SYM_ENTRY *DerefSym(HSym sym);

// Returns 'folder\name' in str (should be 8+8+1+1 bytes long), returns 0 if pb
int HSYMtoName(HSym Sym, char *str);

// Finds the sym and returns a pointer to the folder entry, NULL if pb
//  you should pass n = 0 (don't know yet the use)
SYM_ENTRY * SymFindPtr(char * Name, int n);


#endif
