/******************
* EVENT FUNCTIONS *
******************/

//
// Send the <size> bytes big event <data> to the process with the ID <target>.
//
void TriggerEvent(HANDLE target, WORD size, void *data)
{
  PROCESS_CONTEXT_ENTRY *context = (PROCESS_CONTEXT_ENTRY *)HeapDeref(target);
  RBUFFER *buffer = HeapDeref(context->eventbufferhandle);
  int i;

  // First we write thye size of the event to the buffer...
  RBufferWrite(buffer, (size) >> 8);
  RBufferWrite(buffer, (size) & 0xff);

  // ...then the actual data...
  for(i=0 ; i<size ; i++)
    RBufferWrite(buffer, ((BYTE *)data)[i]);

  // ...and finally the ID of the process that sent it.
  RBufferWrite(buffer, (CurrentProcessID) >> 8);
  RBufferWrite(buffer, (CurrentProcessID) & 0xff);

  // Increase the number of pending events.
  buffer->pending++;

//    ToggleScheduler();
//    asm(" move.l #0xffffffff,(0x4c00)");
//    asm(" l93: bra l93");
//    ToggleScheduler();

  // Wake up the process if it's sleeping/waiting  for events.
  if(context->flags.bits.sleeping) {
    ToggleScheduler();
    MoveProcess(context,&SleepingContextList,&ContextList);
    context->flags.bits.sleeping = 0;
    ToggleScheduler();
  }
//    WakeUp(target);
}

//
// Send the <size> bytes big event <data> to all process started by me.
//
void BroadcastEvent(WORD size, void *data)
{
  SEMAPHORE *cursem;
  PROCESS_CONTEXT_ENTRY *current;

  // First we send the event to all running processes...
  if(current = (PROCESS_CONTEXT_ENTRY *)ContextList.first) {
    do {
      if(current->parent == CurrentProcessID)
        TriggerEvent(current->header.id,size,data);
    } while((current = (PROCESS_CONTEXT_ENTRY *)current->header.next) != (PROCESS_CONTEXT_ENTRY *)ContextList.first);
  }

  // ...then to all sleeping processes...
  if(current = (PROCESS_CONTEXT_ENTRY *)SleepingContextList.first) {
    do {
      if(current->parent == CurrentProcessID)
        TriggerEvent(current->header.id,size,data);
    } while((current = (PROCESS_CONTEXT_ENTRY *)current->header.next) != (PROCESS_CONTEXT_ENTRY *)SleepingContextList.first);
  }

  // ...and finally to all processes that are waiting in semaphores.
  if(cursem = (SEMAPHORE *)SemaphoreList.first) {
    do {
      if(current = (PROCESS_CONTEXT_ENTRY *)(cursem->list.first)) {
        do {
          if(current->parent == CurrentProcessID)
            TriggerEvent(current->header.id,size,data);
        } while((current = (PROCESS_CONTEXT_ENTRY *)(current->header.next)) != (PROCESS_CONTEXT_ENTRY *)(cursem->list.first));
      }
    } while((cursem = (SEMAPHORE *)cursem->header.next) != (SEMAPHORE *)SemaphoreList.first);
  }
}

//
// Check how many events are pending for me.
//
int PendingEvents(void)
{
  RBUFFER *buffer = HeapDeref(CurrentProcessContext->eventbufferhandle);
  return(buffer->pending);
}

//
// Copy the next event on my queue to <dest>. If there isn't any event waiting for
// me then send me to sleep until there is.
//
HANDLE ReadEvent(void *dest)
{
  RBUFFER *buffer = HeapDeref(CurrentProcessContext->eventbufferhandle);
  int size;

  // Send process to sleep if there are bo events at the moment.
  if(buffer->pending == 0) {
    ToggleScheduler();
    CurrentProcessContext->flags.bits.sleeping = 1;
    MoveProcess(CurrentProcessContext,&ContextList,&SleepingContextList);
  }

  // First we get the size of the event...
  size = (RBufferRead(buffer) << 8) + RBufferRead(buffer);

  // ...then the event itself...
  for(; size>0 ; size--) {
    *(BYTE *)(dest++) = RBufferRead(buffer);
  }

  // Decrease the number of pending events.
  buffer->pending--;

  // Finally we read and return the ID of the process that sent the event.
  return (RBufferRead(buffer) << 8) + RBufferRead(buffer);
}
