/******************
* Mouse functions *
******************/

BOOL MouseInside(PWINDOW *w, SCR_RECT *r)
{
  int dX = MouseX - w->X;
  int dY = MouseY - w->Y;

  return((dX >= r->xy.x0) && (dY >= r->xy.y0) && 
         (dX <= r->xy.x1) && (dY <= r->xy.y1));
}

//
// Plot the mouse on the screen.
//
void PlotMouse(void)
{
  MousePlotX = MouseX - MouseHotspotX;
  MousePlotY = MouseY - MouseHotspotY;

  if(MousePlotY > LCD_HEIGHT - MouseHeight) {
    MouseGFXOffset = 0;
    MouseVisibleRows = LCD_HEIGHT - MousePlotY - 1;
    MouseLCDOffset = 30*MousePlotY; // + ((MousePlotX >> 4) << 1);
  }
  else if(MousePlotY < 0) {
    MouseGFXOffset = (-MousePlotY) << 1;
    MouseVisibleRows = MouseHeight + MousePlotY - 1;
    MouseLCDOffset = 0;
//    MouseLCDOffset = ((MousePlotX >> 4) << 1);
  }
  else {
    MouseGFXOffset = 0;
    MouseVisibleRows = MouseHeight - 1;
    MouseLCDOffset = 30*MousePlotY; // + ((MousePlotX >> 4) << 1);
  }

  if(MousePlotX >= (LCD_WIDTH - 16)) {
    MousePlotMask = 0xffff0000;
    MouseLCDOffset += ((MousePlotX >> 4) << 1);
  }
  else if(MousePlotX < 0) {
    MousePlotMask = 0x0000ffff;
    MouseLCDOffset -= (((15-MousePlotX) >> 4) << 1);
  }
  else {
    MousePlotMask = 0xffffffff;
    MouseLCDOffset += ((MousePlotX >> 4) << 1);
  }


  // Draw the mousepointer.
  asm("	MousePlotLoop:
		moveq		#0,%%d6
		move.w		0x40(%4),%%d6	/* Mask data */
		rol.l		%0,%%d6		/* Shift Mask data */
		and.l		MousePlotMask,%%d6

		moveq		#0,%%d5
		move.w		0x20(%4),%%d5	/* GFX data */
		lsl.l		%0,%%d5		/* Shift GFX data */
		and.l		%%d6,%%d5	/* Mask GFX data */

		moveq		#0,%%d4
		move.w		(%4)+,%%d4	/* GFX data */
		lsl.l		%0,%%d4		/* Shift GFX data */
		and.l		%%d6,%%d4	/* Mask GFX data */

		not.l		%%d6
		move.l		(%2),(%5)+	/* Store old LCD data in buffer */
		move.l		(%3),(%5)+	/* Store old LCD data in buffer */
		move.l		%%d6,(%5)+	/* Store shifted mask in buffer */
		and.l		%%d6,(%2)	/* Old LCD data AND mask */
		and.l		%%d6,(%3)	/* Old LCD data AND mask */

		or.l		%%d4,(%2)	/* Add masked GFX data */
		or.l		%%d5,(%3)	/* Add masked GFX data */

		add.w		#30,%2		/* Move to next LCD row */
		add.w		#30,%3		/* Move to next LCD row */
		dbra		%1,MousePlotLoop" : :
		"d" (16 - (MousePlotX & 0x000f)),
		"d" (MouseVisibleRows),
		"a" ((void *)LCD_ADDRESS + MouseLCDOffset),
		"a" ((void *)Plane2 + MouseLCDOffset),
		"a" ((void *)MouseGFX + MouseGFXOffset),
		"a" ((void *)&MouseLCDAndMaskBuffer + 6*MouseGFXOffset) :
		"d4", "d5", "d6", "1", "2", "3", "4", "5");
 }

//
// Remove the mouse from the screen.
//
void RemoveMouse(void)
{
  asm("		move.l		%4,%%d6
		not.l		%%d6
	MouseRemoveLoop:
		move.l		(%3)+,%%d7
		and.l		%4,%%d7
		and.l		%%d6,(%1)
		or.l		%%d7,(%1)

		move.l		(%3)+,%%d7
		and.l		%4,%%d7
		and.l		%%d6,(%2)
		or.l		%%d7,(%2)

		addq.l		#4,%3

		add.l		#30,%1
		add.l		#30,%2
		dbra		%0,MouseRemoveLoop" : :
		"d" (MouseVisibleRows),
		"a" ((void *)LCD_ADDRESS + MouseLCDOffset),
		"a" ((void *)Plane2 + MouseLCDOffset),
		"a" ((void *)&MouseLCDAndMaskBuffer + 6*MouseGFXOffset),
		"d" (MousePlotMask) : "0", "1", "2", "3", "d6", "d7");
}


void MouseInterrupt_Function(void)
{
  if(ARROWKEYSMASK - (ControlKeys.data & ARROWKEYSMASK)) {
    if((MouseDelay--) == 0) {
      MouseX += ((!ControlKeys.flags.right) && (MouseX < LCD_WIDTH-1)) -
                ((!ControlKeys.flags.left) && (MouseX > 0));
      MouseY += ((!ControlKeys.flags.down) && (MouseY < LCD_HEIGHT-1)) -
                ((!ControlKeys.flags.up) && (MouseY > 0));
      MouseDelay = MouseDelays[CurrentMouseDelayIndex];
      if(CurrentMouseDelayIndex < MOUSE_DELAYS-1)
        CurrentMouseDelayIndex++;
    }
  }
  else
    MouseDelay = MouseDelays[CurrentMouseDelayIndex = 0];
}


//
// Activate the mouse. Plot it if it was disabled.
//
void ActivateMouse(void)
{
  while(MouseState == M_PLOTTING)
    BreakAndCooperate();
  if(MouseState == M_DISABLED)
    PlotMouse();
  MouseState = M_ACTIVE;
}

//
// Deactivate the mouse.
//
void DeactivateMouse(void)
{
  while(MouseState == M_PLOTTING)
    BreakAndCooperate();
  MouseState = M_INACTIVE;
}

//
// Disable the mouse, i.e. deactivate it *and* remove it.
//
void DisableMouse(void)
{
  if(MouseState != M_DISABLED) {
    while(MouseState == M_PLOTTING)
      BreakAndCooperate();
    DeactivateMouse();
    RemoveMouse();
    MouseState = M_DISABLED;
  }
}



//
// The Mouse process
//
BEGIN_PROSIT_HEADER(Mouse_Header)
  DEFINE_PROSIT_VERSION(PROSIT_VERSION_NUMBER)
  DEFINE_PROSIT_FILETYPE(PROSIT_FILETYPE_PROGRAM)
  DEFINE_PROSIT_STACKSIZE(1024)
END_PROSIT_HEADER

void Mouse_Process(void)
{
  int OldMouseX = MouseX, OldMouseY = MouseY;

  while(1) {
    if((MouseState == M_ACTIVE) && ((OldMouseX != MouseX) || (OldMouseY != MouseY))) {
      OldMouseX = MouseX;
      OldMouseY = MouseY;

      MouseState = M_PLOTTING;
      RemoveMouse();
      PlotMouse();
      MouseState = M_ACTIVE;
    }
    BreakAndCooperate();
  }
}
